@@ -62,12 +62,11 @@ int
 main(int argc, char *argv[])
 {
     int sock, i;
-    unsigned int len;
+    socklen_t len;
     int flags = 0;                      /* flags for sendto() */
     struct servent *serv;
     struct hostent *host;
     char *cp;
-    char full_hname[MAXHOSTNAMELEN];
 #ifdef BROKEN_STREAMS_SOCKETS
     char my_hostname[MAXHOSTNAMELEN];
 #endif
@@ -145,13 +144,6 @@ main(int argc, char *argv[])
         fprintf(stderr, "%s: unknown host\n", hostname);
         exit(1);
     }
-    strncpy(full_hname, host->h_name, sizeof(full_hname)-1);
-    full_hname[sizeof(full_hname)-1] = '\0';
-
-    /* lower-case to get name for "instance" part of service name */
-    for (cp = full_hname; *cp; cp++)
-        if (isupper((int) *cp))
-            *cp = tolower((int) *cp);
 
     /* Set server's address */
     (void) memset(&s_sock, 0, sizeof(s_sock));
@@ -212,7 +204,7 @@ main(int argc, char *argv[])
         exit(1);
     }
 
-    if ((retval = krb5_mk_req(context, &auth_context, 0, service, full_hname,
+    if ((retval = krb5_mk_req(context, &auth_context, 0, service, hostname,
                               &inbuf, ccdef, &packet))) {
         com_err(progname, retval, "while preparing AP_REQ");
         exit(1);
