@@ -4,8 +4,6 @@
  *               handle the [appdefaults] profile section
  */
 
-#include <stdio.h>
-#include <string.h>
 #include "k5-int.h"
 
 
