@@ -72,7 +72,6 @@
 #ifdef __GNUC__
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wuninitialized"
-#pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
 #endif
 
 #include "k5-int.h"
@@ -153,7 +152,7 @@ static int mylex(int *intp, struct param *tmv);
 static int yyparse(struct param *);
 
 
-#line 157 "deltat.c" /* yacc.c:339  */
+#line 156 "deltat.c" /* yacc.c:339  */
 
 # ifndef YY_NULLPTR
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -185,9 +184,9 @@ extern int yydebug;
 # define YYTOKENTYPE
   enum yytokentype
   {
-    NUM = 258,
-    LONGNUM = 259,
-    OVERFLOW = 260,
+    tok_NUM = 258,
+    tok_LONGNUM = 259,
+    tok_OVERFLOW = 260,
     tok_WS = 261
   };
 #endif
@@ -197,10 +196,10 @@ extern int yydebug;
 typedef union YYSTYPE YYSTYPE;
 union YYSTYPE
 {
-#line 129 "x-deltat.y" /* yacc.c:355  */
+#line 128 "x-deltat.y" /* yacc.c:355  */
 int val;
 
-#line 204 "deltat.c" /* yacc.c:355  */
+#line 203 "deltat.c" /* yacc.c:355  */
 };
 # define YYSTYPE_IS_TRIVIAL 1
 # define YYSTYPE_IS_DECLARED 1
@@ -214,7 +213,7 @@ int yyparse (struct param *tmv);
 
 /* Copy the second part of user declarations.  */
 
-#line 218 "deltat.c" /* yacc.c:358  */
+#line 217 "deltat.c" /* yacc.c:358  */
 
 #ifdef short
 # undef short
@@ -512,8 +511,8 @@ static const yytype_uint8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,   143,   143,   144,   144,   145,   145,   146,   146,   147,
-     148,   150,   151,   152,   153,   154,   155,   156,   157,   161,
+       0,   142,   142,   143,   143,   144,   144,   145,   145,   146,
+     147,   149,   150,   151,   152,   153,   154,   155,   156,   161,
      162,   165,   166,   169,   170
 };
 #endif
@@ -523,8 +522,8 @@ static const yytype_uint8 yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "NUM", "LONGNUM", "OVERFLOW", "'-'",
-  "':'", "'d'", "'h'", "'m'", "'s'", "tok_WS", "$accept", "start",
+  "$end", "error", "$undefined", "tok_NUM", "tok_LONGNUM", "tok_OVERFLOW",
+  "'-'", "':'", "'d'", "'h'", "'m'", "'s'", "tok_WS", "$accept", "start",
   "posnum", "num", "ws", "wsnum", "deltat", "opt_hms", "opt_ms", "opt_s", YY_NULLPTR
 };
 #endif
@@ -1310,93 +1309,93 @@ yyreduce:
   switch (yyn)
     {
         case 6:
-#line 145 "x-deltat.y" /* yacc.c:1646  */
+#line 144 "x-deltat.y" /* yacc.c:1646  */
     { (yyval.val) = - (yyvsp[0].val); }
-#line 1316 "deltat.c" /* yacc.c:1646  */
+#line 1315 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 9:
-#line 147 "x-deltat.y" /* yacc.c:1646  */
+#line 146 "x-deltat.y" /* yacc.c:1646  */
     { (yyval.val) = (yyvsp[0].val); }
-#line 1322 "deltat.c" /* yacc.c:1646  */
+#line 1321 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 10:
-#line 148 "x-deltat.y" /* yacc.c:1646  */
+#line 147 "x-deltat.y" /* yacc.c:1646  */
     { YYERROR; }
-#line 1328 "deltat.c" /* yacc.c:1646  */
+#line 1327 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 11:
-#line 150 "x-deltat.y" /* yacc.c:1646  */
+#line 149 "x-deltat.y" /* yacc.c:1646  */
     { DO ((yyvsp[-2].val),  0,  0, (yyvsp[0].val)); }
-#line 1334 "deltat.c" /* yacc.c:1646  */
+#line 1333 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 12:
-#line 151 "x-deltat.y" /* yacc.c:1646  */
+#line 150 "x-deltat.y" /* yacc.c:1646  */
     { DO ( 0, (yyvsp[-2].val),  0, (yyvsp[0].val)); }
-#line 1340 "deltat.c" /* yacc.c:1646  */
+#line 1339 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 13:
-#line 152 "x-deltat.y" /* yacc.c:1646  */
+#line 151 "x-deltat.y" /* yacc.c:1646  */
     { DO ( 0,  0, (yyvsp[-2].val), (yyvsp[0].val)); }
-#line 1346 "deltat.c" /* yacc.c:1646  */
+#line 1345 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 14:
-#line 153 "x-deltat.y" /* yacc.c:1646  */
+#line 152 "x-deltat.y" /* yacc.c:1646  */
     { DO ( 0,  0,  0, (yyvsp[-1].val)); }
-#line 1352 "deltat.c" /* yacc.c:1646  */
+#line 1351 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 15:
-#line 154 "x-deltat.y" /* yacc.c:1646  */
+#line 153 "x-deltat.y" /* yacc.c:1646  */
     { DO ((yyvsp[-6].val), (yyvsp[-4].val), (yyvsp[-2].val), (yyvsp[0].val)); }
-#line 1358 "deltat.c" /* yacc.c:1646  */
+#line 1357 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 16:
-#line 155 "x-deltat.y" /* yacc.c:1646  */
+#line 154 "x-deltat.y" /* yacc.c:1646  */
     { DO ( 0, (yyvsp[-4].val), (yyvsp[-2].val), (yyvsp[0].val)); }
-#line 1364 "deltat.c" /* yacc.c:1646  */
+#line 1363 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 17:
-#line 156 "x-deltat.y" /* yacc.c:1646  */
+#line 155 "x-deltat.y" /* yacc.c:1646  */
     { DO ( 0, (yyvsp[-2].val), (yyvsp[0].val),  0); }
-#line 1370 "deltat.c" /* yacc.c:1646  */
+#line 1369 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 18:
-#line 157 "x-deltat.y" /* yacc.c:1646  */
+#line 156 "x-deltat.y" /* yacc.c:1646  */
     { DO ( 0,  0,  0, (yyvsp[0].val)); }
-#line 1376 "deltat.c" /* yacc.c:1646  */
+#line 1375 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 20:
 #line 162 "x-deltat.y" /* yacc.c:1646  */
     { if (HOUR_NOT_OK((yyvsp[-2].val))) YYERROR;
 	                                  DO_SUM((yyval.val), (yyvsp[-2].val) * 3600, (yyvsp[0].val)); }
-#line 1383 "deltat.c" /* yacc.c:1646  */
+#line 1382 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 22:
 #line 166 "x-deltat.y" /* yacc.c:1646  */
     { if (MIN_NOT_OK((yyvsp[-2].val))) YYERROR;
 	                                  DO_SUM((yyval.val), (yyvsp[-2].val) * 60, (yyvsp[0].val)); }
-#line 1390 "deltat.c" /* yacc.c:1646  */
+#line 1389 "deltat.c" /* yacc.c:1646  */
     break;
 
   case 23:
 #line 169 "x-deltat.y" /* yacc.c:1646  */
     { (yyval.val) = 0; }
-#line 1396 "deltat.c" /* yacc.c:1646  */
+#line 1395 "deltat.c" /* yacc.c:1646  */
     break;
 
 
-#line 1400 "deltat.c" /* yacc.c:1646  */
+#line 1399 "deltat.c" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -1664,14 +1663,14 @@ mylex(int *intp, struct param *tmv)
 	num = c - '0';
 	while (isdigit ((int) *P)) {
 	  if (num > MAX_TIME / 10)
-	    return OVERFLOW;
+	    return tok_OVERFLOW;
 	    num *= 10;
 	    if (num > MAX_TIME - (*P - '0'))
-	      return OVERFLOW;
+	      return tok_OVERFLOW;
 	    num += *P++ - '0';
 	}
 	*intp = num;
-	return (P - orig_p > 2) ? LONGNUM : NUM;
+	return (P - orig_p > 2) ? tok_LONGNUM : tok_NUM;
     case ' ':
     case '\t':
     case '\n':
