@@ -1,7 +1,6 @@
 /* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
+/* lib/krb5/os/krbfileio.c */
 /*
- * lib/krb5/os/krbfileio.c
- *
  * Copyright (c) Hewlett-Packard Company 1991
  * Released to the Massachusetts Institute of Technology for inclusion
  * in the Kerberos source code distribution.
@@ -27,10 +26,6 @@
  * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
- *
- *
- * krb5_create_secure_file
- * krb5_sync_disk_file
  */
 
 #ifdef MODULE_VERSION_ID
@@ -39,6 +34,7 @@ static char *VersionID = "@(#)krbfileio.c       2 - 08/22/91";
 
 
 #include "k5-int.h"
+#include "os-proto.h"
 #ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif
@@ -53,7 +49,7 @@ static char *VersionID = "@(#)krbfileio.c       2 - 08/22/91";
 #endif
 
 krb5_error_code
-krb5_create_secure_file(krb5_context context, const char *pathname)
+k5_create_secure_file(krb5_context context, const char *pathname)
 {
     int fd;
 
@@ -90,7 +86,7 @@ krb5_create_secure_file(krb5_context context, const char *pathname)
 }
 
 krb5_error_code
-krb5_sync_disk_file(krb5_context context, FILE *fp)
+k5_sync_disk_file(krb5_context context, FILE *fp)
 {
     fflush(fp);
 #if !defined(MSDOS_FILESYSTEM)
