@@ -1,8 +1,7 @@
 /* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
-#include <stdio.h>
-#include <string.h>
-#include <sys/types.h>
+#include "k5-platform.h"
 #include "port-sockets.h"
+#include <sys/types.h>
 #include <com_err.h>
 
 #define TEST
@@ -19,7 +18,7 @@ enum {
 
 const char *prog;
 
-struct addrlist al;
+struct serverlist sl;
 
 static void
 kfatal (krb5_error_code err)
@@ -29,18 +28,20 @@ kfatal (krb5_error_code err)
 }
 
 static const char *
-stypename (int stype)
+ttypename (k5_transport ttype)
 {
     static char buf[20];
-    switch (stype) {
-    case SOCK_STREAM:
-        return "stream";
-    case SOCK_DGRAM:
-        return "dgram";
-    case SOCK_RAW:
-        return "raw";
+    switch (ttype) {
+    case TCP_OR_UDP:
+        return "tcp or udp";
+    case TCP:
+        return "tcp";
+    case UDP:
+        return "udp";
+    case HTTPS:
+        return "https";
     default:
-        snprintf(buf, sizeof(buf), "?%d", stype);
+        snprintf(buf, sizeof(buf), "?%d", ttype);
         return buf;
     }
 }
@@ -48,25 +49,31 @@ stypename (int stype)
 static void
 print_addrs (void)
 {
-    int i;
+    size_t i;
+    int err;
 
-    int naddrs = al.naddrs;
-
-    printf ("%d addresses:\n", naddrs);
-    for (i = 0; i < naddrs; i++) {
-        int err;
-        struct addrinfo *ai = al.addrs[i].ai;
+    printf("%d servers:\n", (int)sl.nservers);
+    for (i = 0; i < sl.nservers; i++) {
+        struct server_entry *entry = &sl.servers[i];
         char hostbuf[NI_MAXHOST], srvbuf[NI_MAXSERV];
-        err = getnameinfo (ai->ai_addr, ai->ai_addrlen,
-                           hostbuf, sizeof (hostbuf),
-                           srvbuf, sizeof (srvbuf),
-                           NI_NUMERICHOST | NI_NUMERICSERV);
-        if (err)
-            printf ("%2d: getnameinfo returns error %d=%s\n",
-                    i, err, gai_strerror (err));
-        else
-            printf ("%2d: address %s\t%s\tport %s\n", i, hostbuf,
-                    stypename (ai->ai_socktype), srvbuf);
+
+        if (entry->hostname != NULL) {
+            printf("%d: h:%s t:%s p:%d m:%d P:%s\n", (int)i,
+                   entry->hostname, ttypename(entry->transport),
+                   entry->port, entry->master,
+                   entry->uri_path ? entry->uri_path : "");
+            continue;
+        }
+        err = getnameinfo((struct sockaddr *)&entry->addr, entry->addrlen,
+                          hostbuf, sizeof(hostbuf), srvbuf, sizeof(srvbuf),
+                          NI_NUMERICHOST | NI_NUMERICSERV);
+        if (err) {
+            printf("%2d: getnameinfo returns error %d=%s\n", (int)i, err,
+                   gai_strerror(err));
+        } else {
+            printf("%2d: address %s\t%s\tport %s\n", (int)i, hostbuf,
+                   ttypename(entry->transport), srvbuf);
+        }
     }
 }
 
@@ -116,22 +123,21 @@ main (int argc, char *argv[])
 
     switch (how) {
     case LOOKUP_CONF:
-        err = krb5_locate_srv_conf (ctx, &realm, "kdc", &al,
-                                    htons (88), htons (750));
+        err = krb5_locate_srv_conf(ctx, &realm, "kdc", &sl, htons(88));
         break;
 
     case LOOKUP_DNS:
-        err = locate_srv_dns_1 (&realm, "_kerberos", "_udp", &al, 0);
+        err = locate_srv_dns_1(ctx, &realm, "_kerberos", "_udp", &sl);
         break;
 
     case LOOKUP_WHATEVER:
-        err = krb5_locate_kdc (ctx, &realm, &al, master, 0, 0);
+        err = k5_locate_kdc(ctx, &realm, &sl, master, FALSE);
         break;
     }
     if (err) kfatal (err);
-    print_addrs ();
+    print_addrs();
 
-    krb5int_free_addrlist (&al);
-    krb5_free_context (ctx);
+    k5_free_serverlist(&sl);
+    krb5_free_context(ctx);
     return 0;
 }
