@@ -1,6 +1,6 @@
+/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
+/* lib/krb5/os/unlck_file.c */
 /*
- * lib/krb5/os/unlck_file.c
- *
  * Copyright 1990 by the Massachusetts Institute of Technology.
  * All Rights Reserved.
  *
@@ -8,7 +8,7 @@
  *   require a specific license from the United States Government.
  *   It is the responsibility of any person or organization contemplating
  *   export to obtain such a license before exporting.
- * 
+ *
  * WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
  * distribute this software and its documentation for any purpose and
  * without fee is hereby granted, provided that the above copyright
@@ -22,18 +22,13 @@
  * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
- * 
- *
- * libos: krb5_lock_file routine
  */
 
 #include "k5-int.h"
 #include <stdio.h>
 
 krb5_error_code
-krb5_unlock_file(context, fd)
-    krb5_context context;
-    int fd;
+krb5_unlock_file(krb5_context context, int fd)
 {
     return krb5_lock_file(context, fd, KRB5_LOCKMODE_UNLOCK);
 }
