@@ -65,7 +65,8 @@ krb5_rc_register_type(krb5_context context, const krb5_rc_ops *ops)
 }
 
 krb5_error_code
-krb5_rc_resolve_type(krb5_context context, krb5_rcache *idptr, char *type)
+krb5_rc_resolve_type(krb5_context context, krb5_rcache *idptr,
+                     const char *type)
 {
     struct krb5_rc_typelist *t;
     krb5_error_code err;
@@ -146,7 +147,7 @@ krb5_rc_default(krb5_context context, krb5_rcache *idptr)
 
 krb5_error_code
 krb5_rc_resolve_full(krb5_context context, krb5_rcache *idptr,
-                     char *string_name)
+                     const char *string_name)
 {
     char *type;
     char *residual;
