@@ -50,8 +50,8 @@
  * organization.
  */
 #define KRB5_MAJOR_RELEASE 1
-#define KRB5_MINOR_RELEASE 15
-#define KRB5_PATCHLEVEL 5
+#define KRB5_MINOR_RELEASE 16
+#define KRB5_PATCHLEVEL 0
 /* #undef KRB5_RELTAIL */
-#define KRB5_RELDATE "20190107"
-#define KRB5_RELTAG "krb5-1.15.5-final"
+#define KRB5_RELDATE "20171205"
+#define KRB5_RELTAG "krb5-1.16-final"
