@@ -40,7 +40,7 @@
 #endif
 
 #define DEFAULT_TYPE_NAME "DEFAULT"
-#define DEFAULT_SOCKET_FMT KDC_DIR "/%s.socket"
+#define DEFAULT_SOCKET_FMT KDC_RUN_DIR "/%s.socket"
 #define DEFAULT_TIMEOUT 5
 #define DEFAULT_RETRIES 3
 #define MAX_SECRET_LEN 1024
@@ -52,11 +52,13 @@ typedef struct token_type_st {
     int timeout;
     size_t retries;
     krb5_boolean strip_realm;
+    char **indicators;
 } token_type;
 
 typedef struct token_st {
     const token_type *type;
     krb5_data username;
+    char **indicators;
 } token;
 
 typedef struct request_st {
@@ -133,6 +135,7 @@ token_type_free(token_type *type)
     free(type->name);
     free(type->server);
     free(type->secret);
+    profile_free_list(type->indicators);
 }
 
 /* Construct the internal default token type. */
@@ -172,6 +175,8 @@ static krb5_error_code
 token_type_decode(profile_t profile, const char *name, token_type *out)
 {
     char *server = NULL, *name_copy = NULL, *secret = NULL, *pstr = NULL;
+    char **indicators = NULL;
+    const char *keys[4];
     int strip_realm, timeout, retries;
     krb5_error_code retval;
 
@@ -241,18 +246,32 @@ token_type_decode(profile_t profile, const char *name, token_type *out)
     if (retval != 0)
         goto cleanup;
 
+    /* Get the authentication indicators to assert if this token is used. */
+    keys[0] = "otp";
+    keys[1] = name;
+    keys[2] = "indicator";
+    keys[3] = NULL;
+    retval = profile_get_values(profile, keys, &indicators);
+    if (retval == PROF_NO_RELATION)
+        retval = 0;
+    if (retval != 0)
+        goto cleanup;
+
     out->name = name_copy;
     out->server = server;
     out->secret = secret;
     out->timeout = timeout;
     out->retries = retries;
     out->strip_realm = strip_realm;
+    out->indicators = indicators;
     name_copy = server = secret = NULL;
+    indicators = NULL;
 
 cleanup:
     free(name_copy);
     free(server);
     free(secret);
+    profile_free_list(indicators);
     return retval;
 }
 
@@ -321,12 +340,61 @@ cleanup:
     return retval;
 }
 
+/* Free a null-terminated array of strings. */
+static void
+free_strings(char **list)
+{
+    char **p;
+
+    for (p = list; p != NULL && *p != NULL; p++)
+        free(*p);
+    free(list);
+}
+
 /* Free the contents of a single token. */
 static void
 token_free_contents(token *t)
 {
-    if (t != NULL)
-        free(t->username.data);
+    if (t == NULL)
+        return;
+    free(t->username.data);
+    free_strings(t->indicators);
+}
+
+/* Decode a JSON array of strings into a null-terminated list of C strings. */
+static krb5_error_code
+indicators_decode(krb5_context ctx, k5_json_value val, char ***indicators_out)
+{
+    k5_json_array arr;
+    k5_json_value obj;
+    char **indicators;
+    size_t len, i;
+
+    *indicators_out = NULL;
+
+    if (k5_json_get_tid(val) != K5_JSON_TID_ARRAY)
+        return EINVAL;
+    arr = val;
+    len = k5_json_array_length(arr);
+    indicators = calloc(len + 1, sizeof(*indicators));
+    if (indicators == NULL)
+        return ENOMEM;
+
+    for (i = 0; i < len; i++) {
+        obj = k5_json_array_get(arr, i);
+        if (k5_json_get_tid(obj) != K5_JSON_TID_STRING) {
+            free_strings(indicators);
+            return EINVAL;
+        }
+        indicators[i] = strdup(k5_json_string_utf8(obj));
+        if (indicators[i] == NULL) {
+            free_strings(indicators);
+            return ENOMEM;
+        }
+    }
+
+    *indicators_out = indicators;
+    return 0;
 }
 
 /* Decode a single token from a JSON token object. */
@@ -336,7 +404,7 @@ token_decode(krb5_context ctx, krb5_const_principal princ,
 {
     const char *typename = DEFAULT_TYPE_NAME;
     const token_type *type = NULL;
-    char *username = NULL;
+    char *username = NULL, **indicators = NULL;
     krb5_error_code retval;
     k5_json_value val;
     size_t i;
@@ -368,8 +436,19 @@ token_decode(krb5_context ctx, krb5_const_principal princ,
             return retval;
     }
 
+    /* Get the authentication indicators if specified. */
+    val = k5_json_object_get(obj, "indicators");
+    if (val != NULL) {
+        retval = indicators_decode(ctx, val, &indicators);
+        if (retval != 0) {
+            free(username);
+            return retval;
+        }
+    }
+
     out->type = type;
     out->username = string2data(username);
+    out->indicators = indicators;
     return 0;
 }
 
@@ -401,6 +480,8 @@ decode_config_json(const char *config, k5_json_array *out)
 
     /* Decode the config string and make sure it's an array. */
     retval = k5_json_decode((config != NULL) ? config : "[{}]", &val);
+    if (retval != 0)
+        goto error;
     if (k5_json_get_tid(val) != K5_JSON_TID_ARRAY) {
         retval = EINVAL;
         goto error;
@@ -543,6 +624,8 @@ callback(krb5_error_code retval, const krad_packet *rqst,
          const krad_packet *resp, void *data)
 {
     request *req = data;
+    token *tok = &req->tokens[req->index];
+    char *const *indicators;
 
     req->index++;
 
@@ -552,7 +635,10 @@ callback(krb5_error_code retval, const krad_packet *rqst,
     /* If we received an accept packet, success! */
     if (krad_packet_get_code(resp) ==
         krad_code_name2num("Access-Accept")) {
-        req->cb(req->data, retval, otp_response_success);
+        indicators = tok->indicators;
+        if (indicators == NULL)
+            indicators = tok->type->indicators;
+        req->cb(req->data, retval, otp_response_success, indicators);
         request_free(req);
         return;
     }
@@ -565,7 +651,7 @@ callback(krb5_error_code retval, const krad_packet *rqst,
     request_send(req);
 
 error:
-    req->cb(req->data, retval, otp_response_fail);
+    req->cb(req->data, retval, otp_response_fail, NULL);
     request_free(req);
 }
 
@@ -592,7 +678,7 @@ request_send(request *req)
     return;
 
 error:
-    req->cb(req->data, retval, otp_response_fail);
+    req->cb(req->data, retval, otp_response_fail, NULL);
     request_free(req);
 }
 
@@ -613,7 +699,7 @@ otp_state_verify(otp_state *state, verto_ctx *ctx, krb5_const_principal princ,
 
     rqst = calloc(1, sizeof(request));
     if (rqst == NULL) {
-        (*cb)(data, ENOMEM, otp_response_fail);
+        (*cb)(data, ENOMEM, otp_response_fail, NULL);
         return;
     }
     rqst->state = state;
@@ -644,6 +730,6 @@ otp_state_verify(otp_state *state, verto_ctx *ctx, krb5_const_principal princ,
     return;
 
 error:
-    (*cb)(data, retval, otp_response_fail);
+    (*cb)(data, retval, otp_response_fail, NULL);
     request_free(rqst);
 }
