@@ -52,6 +52,7 @@ static int debug = 0;
 static char *srvtab = NULL;
 static char *slave_host;
 static char *realm = NULL;
+static char *def_realm = NULL;
 static char *file = KPROP_DEFAULT_FILE;
 
 /* The Kerberos principal we'll be sending as, initialized in get_tickets. */
@@ -63,7 +64,7 @@ static krb5_address *receiver_addr;
 static const char *port = KPROP_SERVICE;
 static char *dbpathname;
 
-static void parse_args(int argc, char **argv);
+static void parse_args(krb5_context context, int argc, char **argv);
 static void get_tickets(krb5_context context);
 static void usage(void);
 static void open_connection(krb5_context context, char *host, int *fd_out);
@@ -101,7 +102,7 @@ main(int argc, char **argv)
         com_err(argv[0], retval, _("while initializing krb5"));
         exit(1);
     }
-    parse_args(argc, argv);
+    parse_args(context, argc, argv);
     get_tickets(context);
 
     database_fd = open_database(context, file, &database_size);
@@ -113,106 +114,72 @@ main(int argc, char **argv)
     printf(_("Database propagation to %s: SUCCEEDED\n"), slave_host);
     krb5_free_cred_contents(context, my_creds);
     close_database(context, database_fd);
+    krb5_free_default_realm(context, def_realm);
     exit(0);
 }
 
 static void
-parse_args(int argc, char **argv)
+parse_args(krb5_context context, int argc, char **argv)
 {
-    char *word, ch;
-
-    progname = *argv++;
-    while (--argc && (word = *argv++) != NULL) {
-        if (*word != '-') {
-            if (slave_host != NULL)
-                usage();
-            else
-                slave_host = word;
-            continue;
-        }
-        word++;
-        while (word != NULL && (ch = *word++) != '\0') {
-            switch (ch) {
-            case 'r':
-                realm = (*word != '\0') ? word : *argv++;
-                if (realm == NULL)
-                    usage();
-                word = NULL;
-                break;
-            case 'f':
-                file = (*word != '\0') ? word : *argv++;
-                if (file == NULL)
-                    usage();
-                word = NULL;
-                break;
-            case 'd':
-                debug++;
-                break;
-            case 'P':
-                port = (*word != '\0') ? word : *argv++;
-                if (port == NULL)
-                    usage();
-                word = NULL;
-                break;
-            case 's':
-                srvtab = (*word != '\0') ? word : *argv++;
-                if (srvtab == NULL)
-                    usage();
-                word = NULL;
-                break;
-            default:
-                usage();
-            }
+    int c;
+    krb5_error_code ret;
 
+    progname = argv[0];
+    while ((c = getopt(argc, argv, "r:f:dP:s:")) != -1) {
+        switch (c) {
+        case 'r':
+            realm = optarg;
+            break;
+        case 'f':
+            file = optarg;
+            break;
+        case 'd':
+            debug++;
+            break;
+        case 'P':
+            port = optarg;
+            break;
+        case 's':
+            srvtab = optarg;
+            break;
+        default:
+            usage();
         }
     }
-    if (slave_host == NULL)
+    if (argc - optind != 1)
         usage();
+    slave_host = argv[optind];
+
+    if (realm == NULL) {
+        ret = krb5_get_default_realm(context, &def_realm);
+        if (ret) {
+            com_err(progname, errno, _("while getting default realm"));
+            exit(1);
+        }
+        realm = def_realm;
+    }
 }
 
 static void
 get_tickets(krb5_context context)
 {
-    char *def_realm, *server;
+    char *server;
     krb5_error_code retval;
     krb5_keytab keytab = NULL;
     krb5_principal server_princ = NULL;
 
     /* Figure out what tickets we'll be using to send. */
-    retval = krb5_sname_to_principal(context, NULL, NULL, KRB5_NT_SRV_HST,
-                                     &my_principal);
+    retval = sn2princ_realm(context, NULL, KPROP_SERVICE_NAME, realm,
+                            &my_principal);
     if (retval) {
         com_err(progname, errno, _("while setting client principal name"));
         exit(1);
     }
-    if (realm != NULL) {
-        retval = krb5_set_principal_realm(context, my_principal, realm);
-        if (retval) {
-            com_err(progname, errno,
-                    _("while setting client principal realm"));
-            exit(1);
-        }
-    } else if (krb5_is_referral_realm(krb5_princ_realm(context,
-                                                       my_principal))) {
-        /* We're going to use this as a client principal, so it can't have the
-         * referral realm.  Use the default realm instead. */
-        retval = krb5_get_default_realm(context, &def_realm);
-        if (retval) {
-            com_err(progname, errno, _("while getting default realm"));
-            exit(1);
-        }
-        retval = krb5_set_principal_realm(context, my_principal, def_realm);
-        if (retval) {
-            com_err(progname, errno,
-                    _("while setting client principal realm"));
-            exit(1);
-        }
-    }
 
     /* Construct the principal name for the slave host. */
     memset(&creds, 0, sizeof(creds));
-    retval = krb5_sname_to_principal(context, slave_host, KPROP_SERVICE_NAME,
-                                     KRB5_NT_SRV_HST, &server_princ);
+    retval = sn2princ_realm(context, slave_host, KPROP_SERVICE_NAME, realm,
+                            &server_princ);
     if (retval) {
         com_err(progname, errno, _("while setting server principal name"));
         exit(1);
@@ -224,13 +191,6 @@ get_tickets(krb5_context context)
         exit(1);
     }
 
-    /* Fill in the client. */
-    retval = krb5_copy_principal(context, my_principal, &creds.client);
-    if (retval) {
-        com_err(progname, retval, _("while copying client principal"));
-        exit(1);
-    }
-
     if (srvtab != NULL) {
         retval = krb5_kt_resolve(context, srvtab, &keytab);
         if (retval) {
@@ -580,6 +540,7 @@ xmit_database(krb5_context context, krb5_auth_context auth_context,
                 send_size, database_size);
         exit(1);
     }
+    free(inbuf.data);
     free(outbuf.data);
 }
 
