@@ -34,8 +34,9 @@ conf4 = {'libdefaults': {
 # Test with client request and session_enctypes preferring aes128, but
 # aes256 long-term key.
 realm = K5Realm(krb5_conf=conf1, create_host=False, get_creds=False)
-realm.run_kadminl('addprinc -randkey -e aes256-cts:normal server')
-realm.run_kadminl('setstr server session_enctypes aes128-cts,aes256-cts')
+realm.run([kadminl, 'addprinc', '-randkey', '-e', 'aes256-cts', 'server'])
+realm.run([kadminl, 'setstr', 'server', 'session_enctypes',
+           'aes128-cts,aes256-cts'])
 test_kvno(realm, 'aes128-cts-hmac-sha1-96', 'aes256-cts-hmac-sha1-96')
 realm.stop()
 
@@ -43,32 +44,35 @@ realm.stop()
 # because of the difference in default_tgs_enctypes order.  This tests that
 # session_enctypes doesn't change the order in which we negotiate.
 realm = K5Realm(krb5_conf=conf2, create_host=False, get_creds=False)
-realm.run_kadminl('addprinc -randkey -e aes256-cts:normal server')
-realm.run_kadminl('setstr server session_enctypes aes128-cts,aes256-cts')
+realm.run([kadminl, 'addprinc', '-randkey', '-e', 'aes256-cts', 'server'])
+realm.run([kadminl, 'setstr', 'server', 'session_enctypes',
+           'aes128-cts,aes256-cts'])
 test_kvno(realm, 'aes256-cts-hmac-sha1-96', 'aes256-cts-hmac-sha1-96')
 realm.stop()
 
 # Next we use conf3 and try various things.
 realm = K5Realm(krb5_conf=conf3, create_host=False, get_creds=False)
-realm.run_kadminl('addprinc -randkey -e aes256-cts:normal server')
+realm.run([kadminl, 'addprinc', '-randkey', '-e', 'aes256-cts:normal',
+           'server'])
 
 # 3a: Negotiate aes128 session key when principal only has aes256 long-term.
-realm.run_kadminl('setstr server session_enctypes aes128-cts,aes256-cts')
+realm.run([kadminl, 'setstr', 'server', 'session_enctypes',
+           'aes128-cts,aes256-cts'])
 test_kvno(realm, 'aes128-cts-hmac-sha1-96', 'aes256-cts-hmac-sha1-96')
 
 # 3b: Negotiate rc4-hmac session key when principal only has aes256 long-term.
-realm.run_kadminl('setstr server session_enctypes '
-                  'rc4-hmac,aes128-cts,aes256-cts')
+realm.run([kadminl, 'setstr', 'server', 'session_enctypes',
+           'rc4-hmac,aes128-cts,aes256-cts'])
 test_kvno(realm, 'arcfour-hmac', 'aes256-cts-hmac-sha1-96')
 
 # 3c: Test des-cbc-crc default assumption.
-realm.run_kadminl('delstr server session_enctypes')
+realm.run([kadminl, 'delstr', 'server', 'session_enctypes'])
 test_kvno(realm, 'des-cbc-crc', 'aes256-cts-hmac-sha1-96')
 realm.stop()
 
 # Last go: test that we can disable the des-cbc-crc assumption
 realm = K5Realm(krb5_conf=conf4, get_creds=False)
-realm.run_kadminl('addprinc -randkey -e aes256-cts:normal server')
+realm.run([kadminl, 'addprinc', '-randkey', '-e', 'aes256-cts', 'server'])
 test_kvno(realm, 'aes256-cts-hmac-sha1-96', 'aes256-cts-hmac-sha1-96')
 realm.stop()
 
