@@ -199,32 +199,41 @@ static errcode_t parse_std_line(char *line, struct parse_state *state)
 }
 
 /* Open and parse an included profile file. */
-static errcode_t parse_include_file(char *filename, struct parse_state *state)
+static errcode_t parse_include_file(const char *filename,
+                                    struct profile_node *root_section)
 {
     FILE    *fp;
     errcode_t retval = 0;
-    struct parse_state incstate;
+    struct parse_state state;
 
-    /* Create a new state so that fragments are syntactically independent,
-     * sharing the root section with the existing state. */
-    incstate.state = STATE_INIT_COMMENT;
-    incstate.group_level = 0;
-    incstate.root_section = state->root_section;
-    incstate.current_section = NULL;
+    /* Create a new state so that fragments are syntactically independent but
+     * share a root section. */
+    state.state = STATE_INIT_COMMENT;
+    state.group_level = 0;
+    state.root_section = root_section;
+    state.current_section = NULL;
 
     fp = fopen(filename, "r");
     if (fp == NULL)
         return PROF_FAIL_INCLUDE_FILE;
-    retval = parse_file(fp, &incstate, NULL);
+    retval = parse_file(fp, &state, NULL);
     fclose(fp);
     return retval;
 }
 
 /* Return non-zero if filename contains only alphanumeric characters, dashes,
- * and underscores. */
+ * and underscores, or if the filename ends in ".conf" and is not a dotfile. */
 static int valid_name(const char *filename)
 {
     const char *p;
+    size_t len = strlen(filename);
+
+    /* Ignore dotfiles, which might be editor or filesystem artifacts. */
+    if (*filename == '.')
+        return 0;
+
+    if (len >= 5 && !strcmp(filename + len - 5, ".conf"))
+        return 1;
 
     for (p = filename; *p != '\0'; p++) {
         if (!isalnum((unsigned char)*p) && *p != '-' && *p != '_')
@@ -234,11 +243,13 @@ static int valid_name(const char *filename)
 }
 
 /*
- * Include files within dirname.  Only files with names consisting entirely of
- * alphanumeric chracters, dashes, and underscores are included, in order to
- * avoid including editor backup files, .rpmsave files, and the like.
+ * Include files within dirname.  Only files with names ending in ".conf", or
+ * consisting entirely of alphanumeric characters, dashes, and underscores are
+ * included.  This restriction avoids including editor backup files, .rpmsave
+ * files, and the like.
  */
-static errcode_t parse_include_dir(char *dirname, struct parse_state *state)
+static errcode_t parse_include_dir(const char *dirname,
+                                   struct profile_node *root_section)
 {
 #ifdef _WIN32
     char *wildcard = NULL, *pathname;
@@ -262,7 +273,7 @@ static errcode_t parse_include_dir(char *dirname, struct parse_state *state)
             retval = ENOMEM;
             break;
         }
-        retval = parse_include_file(pathname, state);
+        retval = parse_include_file(pathname, root_section);
         free(pathname);
         if (retval)
             break;
@@ -291,7 +302,7 @@ cleanup:
             retval = ENOMEM;
             break;
         }
-        retval = parse_include_file(pathname, state);
+        retval = parse_include_file(pathname, root_section);
         free(pathname);
         if (retval)
             break;
@@ -309,12 +320,12 @@ static errcode_t parse_line(char *line, struct parse_state *state,
     if (strncmp(line, "include", 7) == 0 && isspace(line[7])) {
         cp = skip_over_blanks(line + 7);
         strip_line(cp);
-        return parse_include_file(cp, state);
+        return parse_include_file(cp, state->root_section);
     }
     if (strncmp(line, "includedir", 10) == 0 && isspace(line[10])) {
         cp = skip_over_blanks(line + 10);
         strip_line(cp);
-        return parse_include_dir(cp, state);
+        return parse_include_dir(cp, state->root_section);
     }
     switch (state->state) {
     case STATE_INIT_COMMENT:
@@ -449,6 +460,25 @@ errcode_t profile_parse_file(FILE *f, struct profile_node **root,
     return 0;
 }
 
+errcode_t profile_process_directory(const char *dirname,
+                                    struct profile_node **root)
+{
+    errcode_t retval;
+    struct profile_node *node;
+
+    *root = NULL;
+    retval = profile_create_node("(root)", 0, &node);
+    if (retval)
+        return retval;
+    retval = parse_include_dir(dirname, node);
+    if (retval) {
+        profile_free_node(node);
+        return retval;
+    }
+    *root = node;
+    return 0;
+}
+
 /*
  * Return TRUE if the string begins or ends with whitespace
  */
