@@ -1,6 +1,6 @@
+/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
+/* util/ss/requests.c */
 /*
- * Various minor routines...
- *
  * Copyright 1987, 1988, 1989 by MIT
  *
  * For copyright information, see mit-sipb-copyright.h.
@@ -9,16 +9,16 @@
 #include <stdio.h>
 #include "ss_internal.h"
 
-#define	DECLARE(name)	void name(argc,argv,sci_idx)int argc,sci_idx;char **argv;
+#define DECLARE(name)   void name(argc,argv,sci_idx,info_ptr)int argc,sci_idx;const char * const *argv; pointer info_ptr;
 
 /*
  * ss_self_identify -- assigned by default to the "." request
  */
 DECLARE(ss_self_identify)
 {
-     register ss_data *info = ss_info(sci_idx);
-     printf("%s version %s\n", info->subsystem_name,
-	    info->subsystem_version);
+    register ss_data *info = ss_info(sci_idx);
+    printf("%s version %s\n", info->subsystem_name,
+           info->subsystem_version);
 }
 
 /*
@@ -26,7 +26,7 @@ DECLARE(ss_self_identify)
  */
 DECLARE(ss_subsystem_name)
 {
-     printf("%s\n", ss_info(sci_idx)->subsystem_name);
+    printf("%s\n", ss_info(sci_idx)->subsystem_name);
 }
 
 /*
@@ -34,7 +34,7 @@ DECLARE(ss_subsystem_name)
  */
 DECLARE(ss_subsystem_version)
 {
-     printf("%s\n", ss_info(sci_idx)->subsystem_version);
+    printf("%s\n", ss_info(sci_idx)->subsystem_version);
 }
 
 /*
@@ -43,5 +43,5 @@ DECLARE(ss_subsystem_version)
  */
 DECLARE(ss_unimplemented)
 {
-     ss_perror(sci_idx, SS_ET_UNIMPLEMENTED, "");
+    ss_perror(sci_idx, SS_ET_UNIMPLEMENTED, "");
 }
