@@ -52,7 +52,7 @@
  * the data structures and flag values locally.
  *
  *
- * On Mac OS X, getaddrinfo results aren't cached (though
+ * On macOS, getaddrinfo results aren't cached (though
  * gethostbyname results are), so we need to build a cache here.  Now
  * things are getting really messy.  Because the cache is in use, we
  * use getservbyname, and throw away thread safety.  (Not that the
@@ -331,18 +331,6 @@ system_freeaddrinfo (struct addrinfo *ai)
     freeaddrinfo(ai);
 }
 
-/* Note: Implementations written to RFC 2133 use size_t, while RFC
-   2553 implementations use socklen_t, for the second parameter.
-
-   Mac OS X (10.2) and AIX 4.3.3 appear to be in the RFC 2133 camp,
-   but we don't have an autoconf test for that right now.  */
-static inline int
-system_getnameinfo (const struct sockaddr *sa, socklen_t salen,
-                    char *host, size_t hostlen, char *serv, size_t servlen,
-                    int flags)
-{
-    return getnameinfo(sa, salen, host, hostlen, serv, servlen, flags);
-}
 #endif
 
 #if !defined (HAVE_GETADDRINFO) || defined(WRAP_GETADDRINFO) || defined(FAI_CACHE)
@@ -697,7 +685,7 @@ static inline int fai_add_hosts_by_name (const char *name,
            sometimes associates it with the specified service,
            sometimes not.
 
-           But on Mac OS X (10.3, 10.4) they've "extended" getaddrinfo
+           But on macOS (10.3, 10.4) they've "extended" getaddrinfo
            to make SRV RR queries.  (Please, somebody, show me
            something in the specs that actually supports this?  RFC
            3493 says nothing about it, but it does say getaddrinfo is
