@@ -35,6 +35,8 @@
 #include "cns.h"
 #include "tktlist.h"
 
+#define ts2tt(t) (time_t)(uint32_t)(t)
+
 /*
  * Ticket information for a list line
  */
@@ -99,7 +101,6 @@ ticket_init_list (HWND hwnd)
   krb5_cc_cursor cursor;
   krb5_error_code code;
   krb5_creds c;
-  krb5_flags flags;
   char *sname;                            /* Name of the service */
   char *flags_string(krb5_creds *cred);
 #endif
@@ -156,15 +157,11 @@ ticket_init_list (HWND hwnd)
 #ifdef KRB5
 
   ncred = 0;
-  flags = 0;
-  if (code = krb5_cc_set_flags(k5_context, k5_ccache, flags)) {
+  if (code = krb5_cc_start_seq_get(k5_context, k5_ccache, &cursor)) {
     if (code != KRB5_FCC_NOFILE) {
       return -1;
     }
   } else {
-    if (code = krb5_cc_start_seq_get(k5_context, k5_ccache, &cursor)) {
-      return -1;
-    }
     while (1) {
       code = krb5_cc_next_cred(k5_context, k5_ccache, &cursor, &c);
       if (code != 0)
@@ -172,10 +169,10 @@ ticket_init_list (HWND hwnd)
 
       ncred++;
       strcpy (buf, "  ");
-      strncat(buf, short_date (c.times.starttime - kwin_get_epoch()),
+      strncat(buf, short_date(ts2tt(c.times.starttime) - kwin_get_epoch()),
 	      sizeof(buf) - 1 - strlen(buf));
       strncat(buf, "      ", sizeof(buf) - 1 - strlen(buf));
-      strncat(buf, short_date (c.times.endtime - kwin_get_epoch()),
+      strncat(buf, short_date(ts2tt(c.times.endtime) - kwin_get_epoch()),
 	      sizeof(buf) - 1 - strlen(buf));
       strncat(buf, "      ", sizeof(buf) - 1 - strlen(buf));
 
@@ -197,8 +194,8 @@ ticket_init_list (HWND hwnd)
 	return -1;
 
       lpinfo->ticket = TRUE;
-      lpinfo->issue_time = c.times.starttime - kwin_get_epoch();
-      lpinfo->lifetime = c.times.endtime - c.times.starttime;
+      lpinfo->issue_time = ts2tt(c.times.starttime) - kwin_get_epoch();
+      lpinfo->lifetime = ts2tt(c.times.endtime) - c.times.starttime;
       strcpy(lpinfo->buf, buf);
 
       rc = ListBox_AddItemData(hwnd, lpinfo);
@@ -211,10 +208,6 @@ ticket_init_list (HWND hwnd)
       if (code = krb5_cc_end_seq_get(k5_context, k5_ccache, &cursor)) {
 	return -1;
       }
-      flags = KRB5_TC_OPENCLOSE;          /* turns on OPENCLOSE mode */
-      if (code = krb5_cc_set_flags(k5_context, k5_ccache, flags)) {
-	return -1;
-      }
     } else {
       return -1;
     }
