@@ -66,11 +66,13 @@ BEGIN_MESSAGE_MAP(CLeashView, CListView)
 	ON_COMMAND(ID_RENEWABLE_UNTIL, OnRenewableUntil)
 	ON_COMMAND(ID_SHOW_TICKET_FLAGS, OnShowTicketFlags)
 	ON_COMMAND(ID_ENCRYPTION_TYPE, OnEncryptionType)
+	ON_COMMAND(ID_CCACHE_NAME, OnCcacheName)
 	ON_UPDATE_COMMAND_UI(ID_TIME_ISSUED, OnUpdateTimeIssued)
 	ON_UPDATE_COMMAND_UI(ID_VALID_UNTIL, OnUpdateValidUntil)
 	ON_UPDATE_COMMAND_UI(ID_RENEWABLE_UNTIL, OnUpdateRenewableUntil)
 	ON_UPDATE_COMMAND_UI(ID_SHOW_TICKET_FLAGS, OnUpdateShowTicketFlags)
 	ON_UPDATE_COMMAND_UI(ID_ENCRYPTION_TYPE, OnUpdateEncryptionType)
+	ON_UPDATE_COMMAND_UI(ID_CCACHE_NAME, OnUpdateCcacheName)
 	ON_COMMAND(ID_UPPERCASE_REALM, OnUppercaseRealm)
 	ON_COMMAND(ID_KILL_TIX_ONEXIT, OnKillTixOnExit)
 	ON_UPDATE_COMMAND_UI(ID_UPPERCASE_REALM, OnUpdateUppercaseRealm)
@@ -143,12 +145,13 @@ BOOL CLeashView::m_importedTickets = 0;
 LONG CLeashView::m_timerMsgNotInProgress = 1;
 ViewColumnInfo CLeashView::sm_viewColumns[] =
 {
-    {"Principal", true, -1, 100},                        // PRINCIPAL
+    {"Principal", true, -1, 200},                        // PRINCIPAL
     {"Issued", false, ID_TIME_ISSUED, 100},              // TIME_ISSUED
     {"Renewable Until", false, ID_RENEWABLE_UNTIL, 100}, // RENEWABLE_UNTIL
     {"Valid Until", true, ID_VALID_UNTIL, 100},          // VALID_UNTIL
     {"Encryption Type", false, ID_ENCRYPTION_TYPE, 100}, // ENCRYPTION_TYPE
     {"Flags", false, ID_SHOW_TICKET_FLAGS, 100},         // TICKET_FLAGS
+    {"Credential Cache", false, ID_CCACHE_NAME, 105},    // CACHE_NAME
 };
 
 static struct TicketFlag {
@@ -226,22 +229,22 @@ static HFONT CreateBoldItalicFont(HFONT font)
 
 bool change_icon_size = true;
 
-void krb5TimestampToFileTime(krb5_timestamp t, LPFILETIME pft)
+void TimestampToFileTime(time_t t, LPFILETIME pft)
 {
     // Note that LONGLONG is a 64-bit value
-    LONGLONG ll;
+    ULONGLONG ll;
 
-    ll = Int32x32To64(t, 10000000) + 116444736000000000;
+    ll = UInt32x32To64((DWORD)t, 10000000) + 116444736000000000;
     pft->dwLowDateTime = (DWORD)ll;
     pft->dwHighDateTime = ll >> 32;
 }
 
 // allocate outstr
-void krb5TimestampToLocalizedString(krb5_timestamp t, LPTSTR *outStr)
+void TimestampToLocalizedString(time_t t, LPTSTR *outStr)
 {
     FILETIME ft, lft;
     SYSTEMTIME st;
-    krb5TimestampToFileTime(t, &ft);
+    TimestampToFileTime(t, &ft);
     FileTimeToLocalFileTime(&ft, &lft);
     FileTimeToSystemTime(&lft, &st);
     TCHAR timeFormat[80]; // 80 is max required for LOCALE_STIMEFORMAT
@@ -1122,11 +1125,12 @@ void CLeashView::AddDisplayItem(CListCtrl &list,
                                 CCacheDisplayData *elem,
                                 int iItem,
                                 char *principal,
-                                long issued,
-                                long valid_until,
-                                long renew_until,
+                                time_t issued,
+                                time_t valid_until,
+                                time_t renew_until,
                                 char *encTypes,
-                                unsigned long flags)
+                                unsigned long flags,
+                                char *ccache_name)
 {
     TCHAR* localTimeStr=NULL;
     TCHAR* durationStr=NULL;
@@ -1141,7 +1145,7 @@ void CLeashView::AddDisplayItem(CListCtrl &list,
         if (issued == 0) {
             list.SetItemText(iItem, iSubItem++, "Unknown");
         } else {
-            krb5TimestampToLocalizedString(issued, &localTimeStr);
+            TimestampToLocalizedString(issued, &localTimeStr);
             list.SetItemText(iItem, iSubItem++, localTimeStr);
         }
     }
@@ -1151,7 +1155,7 @@ void CLeashView::AddDisplayItem(CListCtrl &list,
         } else if (valid_until < now) {
             list.SetItemText(iItem, iSubItem++, "Expired");
         } else if (renew_until) {
-            krb5TimestampToLocalizedString(renew_until, &localTimeStr);
+            TimestampToLocalizedString(renew_until, &localTimeStr);
             DurationToString(renew_until - now, &durationStr);
             if (localTimeStr && durationStr) {
                 _snprintf(tempStr, MAX_DURATION_STR, "%s %s", localTimeStr, durationStr);
@@ -1168,7 +1172,7 @@ void CLeashView::AddDisplayItem(CListCtrl &list,
         } else if (valid_until < now) {
             list.SetItemText(iItem, iSubItem++, "Expired");
         } else {
-            krb5TimestampToLocalizedString(valid_until, &localTimeStr);
+            TimestampToLocalizedString(valid_until, &localTimeStr);
             DurationToString(valid_until - now, &durationStr);
             if (localTimeStr && durationStr) {
                 _snprintf(tempStr, MAX_DURATION_STR, "%s %s", localTimeStr, durationStr);
@@ -1185,6 +1189,9 @@ void CLeashView::AddDisplayItem(CListCtrl &list,
         krb5TicketFlagsToString(flags, &flagsStr);
         list.SetItemText(iItem, iSubItem++, flagsStr);
     }
+    if (sm_viewColumns[CACHE_NAME].m_enabled) {
+        list.SetItemText(iItem, iSubItem++, ccache_name);
+    }
     if (flagsStr)
         free(flagsStr);
     if (localTimeStr)
@@ -1477,7 +1484,8 @@ VOID CLeashView::OnUpdateDisplay()
                        principal->valid_until,
                        principal->renew_until,
                        "",
-                       principal->flags);
+                       principal->flags,
+                       principal->ccache_name);
         if (elem->m_expanded) {
             for (tempList = principal->ticket_list;
                  tempList != NULL;
@@ -1490,7 +1498,8 @@ VOID CLeashView::OnUpdateDisplay()
                                tempList->valid_until,
                                tempList->renew_until,
                                tempList->encTypes,
-                               tempList->flags);
+                               tempList->flags,
+                               principal->ccache_name);
             }
         }
         if ((elem->m_focus >= 0) &&
@@ -1803,6 +1812,16 @@ VOID CLeashView::OnUpdateEncryptionType(CCmdUI *pCmdUI)
     pCmdUI->SetCheck(sm_viewColumns[ENCRYPTION_TYPE].m_enabled);
 }
 
+VOID CLeashView::OnCcacheName()
+{
+    ToggleViewColumn(CACHE_NAME);
+}
+
+VOID CLeashView::OnUpdateCcacheName(CCmdUI *pCmdUI)
+{
+    pCmdUI->SetCheck(sm_viewColumns[CACHE_NAME].m_enabled);
+}
+
 VOID CLeashView::OnLargeIcons()
 {
     INT x, y, n;
